@@ -41,7 +41,6 @@
 #include "rb-dialog.h"
 #include "rb-new-station-dialog.h"
 #include "rb-station-properties-dialog.h"
-#include "rb-volume.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 
@@ -90,7 +89,6 @@ static void rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view
 /* source methods */
 static char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
-static const char *impl_get_search_key (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static void impl_search (RBSource *source, const char *text);
@@ -119,7 +117,6 @@ static void stations_view_drag_data_received_cb (GtkWidget *widget,
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/iradio/paned_position"
 #define CONF_STATE_IRADIO_SORTING CONF_PREFIX "/state/iradio/sorting"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/iradio/show_browser"
-#define CONF_STATE_SEARCH_TEXT   CONF_PREFIX "/state/iradio/search_text"
 
 struct RBIRadioSourcePrivate
 {
@@ -209,7 +206,6 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 
 	source_class->impl_get_status  = impl_get_status;
 	source_class->impl_get_browser_key  = impl_get_browser_key;
-	source_class->impl_get_search_key  = impl_get_search_key;
 	source_class->impl_get_pixbuf  = impl_get_pixbuf;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_search = impl_search;
@@ -219,9 +215,10 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 	source_class->impl_song_properties = impl_song_properties;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_handle_eos = impl_handle_eos;
-	source_class->impl_have_artist_album = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_try_playlist = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_have_url = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_show_popup = impl_show_popup;
+	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_false_function;
 
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_TYPE,
@@ -238,7 +235,7 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 static void
 rb_iradio_source_init (RBIRadioSource *source)
 {
-	GtkWidget *dummy = gtk_tree_view_new ();
+	gint size;
 	source->priv = g_new0 (RBIRadioSourcePrivate, 1);
 
 	source->priv->vbox = gtk_vbox_new (FALSE, 5);
@@ -249,11 +246,11 @@ rb_iradio_source_init (RBIRadioSource *source)
 	source->priv->name = g_strdup (_("Unknown"));
 	source->priv->title = g_strdup (_("Unknown"));
 
-	source->priv->pixbuf = gtk_widget_render_icon (dummy,
-						       RB_STOCK_IRADIO,
-						       GTK_ICON_SIZE_LARGE_TOOLBAR,
-						       NULL);
-	gtk_widget_destroy (dummy);
+	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+	source->priv->pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+							 "stock_channel",
+							 size,
+							 0, NULL);
 }
 
 static void
@@ -323,7 +320,7 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_GENRE);
 /* 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_QUALITY); */
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_RATING);
-	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_PLAY_COUNT);
+/*	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_PLAY_COUNT);*/
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_LAST_PLAYED);
 	g_signal_connect_object (G_OBJECT (source->priv->stations),
 				 "sort-order-changed",
@@ -433,7 +430,6 @@ rb_iradio_source_new (RBShell *shell)
 	source = RB_SOURCE (g_object_new (RB_TYPE_IRADIO_SOURCE,
 					  "name", _("Radio"),
 					  "shell", shell,
-					  "internal-name", "<radio>",
 					  NULL));
 
 	rb_shell_register_entry_type_for_source (shell, source, 
@@ -449,40 +445,31 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 	RhythmDBEntry *entry;
 	GValue val = { 0, };
 
-	if (uri && !g_utf8_validate (uri, -1, NULL)) {
-		rb_debug ("uri \"%s\" is not valid UTF-8", uri);
-		return;
-	}
-	if (title && !g_utf8_validate (title, -1, NULL)) {
-		rb_debug ("title \"s\" is not valid UTF-8", title);
-		return;
-	}
-	if (genre && !g_utf8_validate (genre, -1, NULL)) {
-		rb_debug ("genre \"s\" is not valid UTF-8", genre);
-		genre = "";
-	}
-
 	entry = rhythmdb_entry_lookup_by_location (source->priv->db, uri);
-	if (!entry)
-		entry = rhythmdb_entry_new (source->priv->db, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, uri);
+	if (entry) {
+		rb_debug ("uri %s already in db", uri); 
+		return;
+	}
+	entry = rhythmdb_entry_new (source->priv->db, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, uri);
 	g_value_init (&val, G_TYPE_STRING);
 	if (title)
-		g_value_set_string (&val, title);
+		g_value_set_static_string (&val, title);
 	else
-		g_value_set_string (&val, uri);
+		g_value_take_string (&val,
+				     gnome_vfs_format_uri_for_display (uri));
 
-	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_TITLE, &val);
+	rhythmdb_entry_set_uninserted (source->priv->db, entry, RHYTHMDB_PROP_TITLE, &val);
 	g_value_reset (&val);
 		
 	if ((!genre) || (strcmp (genre, "") == 0))
 		genre = _("Unknown");
 	g_value_set_string (&val, genre);
-	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
+	rhythmdb_entry_set_uninserted (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
 	g_value_unset (&val);
 	
 	g_value_init (&val, G_TYPE_DOUBLE);
 	g_value_set_double (&val, 2.5);
-	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_RATING, &val);
+	rhythmdb_entry_set_uninserted (source->priv->db, entry, RHYTHMDB_PROP_RATING, &val);
 	g_value_unset (&val);
 	
 	rhythmdb_commit (source->priv->db);
@@ -515,10 +502,7 @@ impl_search (RBSource *asource, const char *search_text)
 		search_text = NULL;
 
 	g_free (source->priv->search_text);
-	if (search_text)
-		source->priv->search_text = g_utf8_casefold (search_text, -1);
-	else
-		source->priv->search_text = NULL;
+	source->priv->search_text = g_strdup (search_text);
 	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_SEARCH);
 
 	rb_source_notify_filter_changed (RB_SOURCE (source));
@@ -535,7 +519,7 @@ impl_get_entry_view (RBSource *asource)
 static RBSourceEOFType
 impl_handle_eos (RBSource *asource)
 {
-	return RB_SOURCE_EOF_ERROR;
+	return RB_SOURCE_EOF_RETRY;
 }
 
 #if 0
@@ -588,12 +572,6 @@ impl_get_browser_key (RBSource *asource)
 	return CONF_STATE_SHOW_BROWSER;
 }
 
-static const char *
-impl_get_search_key (RBSource *asource)
-{
-	return CONF_STATE_SEARCH_TEXT;
-}
-
 static void
 impl_delete (RBSource *asource)
 {
@@ -791,10 +769,8 @@ static void
 handle_playlist_entry_cb (TotemPlParser *playlist, const char *uri, const char *title,
 			  const char *genre, RBIRadioSource *source)
 {
-	if (rb_uri_is_iradio (uri)) {
-		if (!rhythmdb_entry_lookup_by_location (source->priv->db, uri)) {
-			rb_iradio_source_add_station (source, uri, title, genre);
-		}
+	if (!rhythmdb_entry_lookup_by_location (source->priv->db, uri)) {
+		rb_iradio_source_add_station (source, uri, title, genre);
 	}
 }
 
@@ -843,8 +819,6 @@ stations_view_drag_data_received_cb (GtkWidget *widget,
 
 	for (i = uri_list; i != NULL; i = i->next) {
 		char *uri = NULL;
-		char *title_data = NULL;
-		char *title = NULL;
 
 		uri = i->data;
 
@@ -857,13 +831,11 @@ stations_view_drag_data_received_cb (GtkWidget *widget,
 		}
 
 		if ((uri != NULL) && 
-		    (rb_uri_is_iradio (uri)) &&
 		    (!rhythmdb_entry_lookup_by_location (source->priv->db, uri))) {
-			rb_iradio_source_add_station (source, uri, title, NULL);
+			rb_iradio_source_add_station (source, uri, NULL, NULL);
 		}
 
 		g_free (uri);
-		g_free (title_data);
 	}
 
 	g_list_free (uri_list);
