@@ -41,7 +41,6 @@
 #if defined(WITH_CD_BURNER_SUPPORT)
 #include "rb-playlist-source-recorder.h"
 #endif
-#include "rb-volume.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 #include "rb-song-info.h"
@@ -53,9 +52,11 @@
 #define RB_PLAYLIST_AUTOMATIC (xmlChar *) "automatic"
 #define RB_PLAYLIST_STATIC (xmlChar *) "static"
 #define RB_PLAYLIST_NAME (xmlChar *) "name"
-#define RB_PLAYLIST_INT_NAME (xmlChar *) "internal-name"
 #define RB_PLAYLIST_LIMIT_COUNT (xmlChar *) "limit-count"
 #define RB_PLAYLIST_LIMIT_SIZE (xmlChar *) "limit-size"
+#define RB_PLAYLIST_LIMIT_TIME (xmlChar *) "limit-time"
+#define RB_PLAYLIST_SORT_KEY (xmlChar *) "sort-key"
+#define RB_PLAYLIST_SORT_DIRECTION (xmlChar *) "sort-direction"
 #define RB_PLAYLIST_LIMIT (xmlChar *) "limit"
 #define RB_PLAYLIST_LOCATION (xmlChar *) "location"
 
@@ -88,6 +89,8 @@ static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static gboolean impl_show_popup (RBSource *source);
 static void rb_playlist_source_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry,
 					       RBPlaylistSource *source);
+static void rb_playlist_source_songs_sort_order_changed_cb (RBEntryView *view,
+							    RBPlaylistSource *source);
 
 static void rb_playlist_source_songs_show_popup_cb (RBEntryView *view, RBPlaylistSource *playlist_view);
 static void rb_playlist_source_drop_cb (GtkWidget *widget,
@@ -100,6 +103,18 @@ static void rb_playlist_source_drop_cb (GtkWidget *widget,
 				     gpointer user_data);
 static void rb_playlist_source_add_list_uri (RBPlaylistSource *source,
 					  GList *list);
+static void rb_playlist_source_do_query (RBPlaylistSource *source,
+					 GPtrArray *query,
+					 guint limit_count,
+					 guint limit_mb,
+					 guint limit_time);
+static void rb_playlist_source_row_inserted (GtkTreeModel *treemodel,
+					     GtkTreePath *path,
+					     GtkTreeIter *iter,
+					     RBPlaylistSource *source);
+static void rb_playlist_source_non_entry_dropped (GtkTreeModel *model,
+						  const char *uri,
+						  RBPlaylistSource *source);
 
 #define PLAYLIST_SOURCE_SONGS_POPUP_PATH "/PlaylistViewPopup"
 #define PLAYLIST_SOURCE_POPUP_PATH "/PlaylistSourcePopup"
@@ -115,6 +130,7 @@ struct RBPlaylistSourcePrivate
 	GHashTable *entries;
 
 	RhythmDBQueryModel *model;
+	gboolean query_resetting;
 
 	GtkWidget *vbox;
 	GdkPixbuf *normal_pixbuf;
@@ -123,6 +139,7 @@ struct RBPlaylistSourcePrivate
 	RBEntryView *songs;
 
 	gboolean dirty;
+	gboolean is_local;
 
 	char *title;
 };
@@ -131,7 +148,9 @@ enum
 {
 	PROP_0,
 	PROP_AUTOMATIC,
-	PROP_DIRTY
+	PROP_QUERY_MODEL,
+	PROP_DIRTY,
+	PROP_LOCAL
 };
 
 static GObjectClass *parent_class = NULL;
@@ -195,7 +214,6 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	source_class->impl_delete = impl_delete;
 	source_class->impl_song_properties = impl_song_properties;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_have_artist_album = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_have_url = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_show_popup = impl_show_popup;
@@ -208,6 +226,14 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property (object_class,
+					 PROP_QUERY_MODEL,
+					 g_param_spec_object ("query-model",
+							      "query-model",
+							      "query model for the playlist",
+							      RHYTHMDB_TYPE_QUERY_MODEL,
+							      G_PARAM_READABLE));
+
 	g_object_class_install_property (object_class,
 					 PROP_DIRTY,
 					 g_param_spec_boolean ("dirty",
@@ -215,6 +241,15 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 							       "whether this playlist should be saved",
 							       FALSE,
 							       G_PARAM_READABLE));
+
+	g_object_class_install_property (object_class,
+					 PROP_LOCAL,
+					 g_param_spec_boolean ("is-local",
+							       "is-local",
+							       "whether this playlist is attached to the local library",
+							       FALSE,
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 }
 
 static void
@@ -244,7 +279,6 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 	RBPlaylistSource *source;
 	RBPlaylistSourceClass *klass;
 	GObjectClass *parent_class;  
-	GtkWidget *dummy = gtk_tree_view_new ();
 	RBShell *shell;
 
 	klass = RB_PLAYLIST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PLAYLIST_SOURCE));
@@ -274,6 +308,14 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 		
 	source->priv->songs = rb_entry_view_new (source->priv->db, NULL, TRUE, TRUE);
 
+	/* watch these to find out when things were dropping into the entry view */
+	g_signal_connect_object (G_OBJECT (source->priv->model), "row-inserted",
+			 G_CALLBACK (rb_playlist_source_row_inserted),
+			 source, 0);
+	g_signal_connect_object (G_OBJECT (source->priv->model), "non-entry-dropped",
+			 G_CALLBACK (rb_playlist_source_non_entry_dropped),
+			 source, 0);
+
 	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_QUERY_MODEL (source->priv->model));
 
 	{
@@ -298,26 +340,36 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_RATING);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_PLAY_COUNT);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LAST_PLAYED);
-	rb_entry_view_set_columns_clickable (source->priv->songs, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_FIRST_SEEN);
 
 	g_signal_connect_object (G_OBJECT (source->priv->songs), "show_popup",
 				 G_CALLBACK (rb_playlist_source_songs_show_popup_cb), source, 0);
+	g_signal_connect_object (G_OBJECT (source->priv->songs), "sort-order-changed",
+				 G_CALLBACK (rb_playlist_source_songs_sort_order_changed_cb), source, 0);
 		
 	g_signal_connect_object (G_OBJECT (source->priv->songs), "drag_data_received",
 				 G_CALLBACK (rb_playlist_source_drop_cb), source, 0);
 	gtk_drag_dest_set (GTK_WIDGET (source->priv->songs), GTK_DEST_DEFAULT_ALL,
 			   target_uri, G_N_ELEMENTS (target_uri), GDK_ACTION_COPY);
+
+	{
+		GtkIconTheme *theme = gtk_icon_theme_get_default();
+		gint size;
+
+		gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+		source->priv->normal_pixbuf = gtk_icon_theme_load_icon (theme,
+									"stock_playlist",
+									size,
+									0, NULL);
+		source->priv->smartypants_pixbuf = gtk_icon_theme_load_icon (theme,
+									     "stock_smart-playlist",
+									     size,
+									     0, NULL);
+	}
 		
-	source->priv->normal_pixbuf = gtk_widget_render_icon (dummy,
-						       RB_STOCK_PLAYLIST,
-						       GTK_ICON_SIZE_LARGE_TOOLBAR,
-						       NULL);
-	source->priv->smartypants_pixbuf = gtk_widget_render_icon (dummy,
-								   RB_STOCK_AUTOMATIC_PLAYLIST,
-								   GTK_ICON_SIZE_LARGE_TOOLBAR,
-								   NULL);
-	gtk_widget_destroy (dummy);
-		
+	rb_entry_view_set_columns_clickable (source->priv->songs, FALSE);
+	source->priv->query_resetting = FALSE;
+
 	gtk_box_pack_start_defaults (GTK_BOX (source->priv->vbox), GTK_WIDGET (source->priv->songs));
 		
 	gtk_widget_show_all (GTK_WIDGET (source));
@@ -386,6 +438,9 @@ rb_playlist_source_set_property (GObject *object,
 	case PROP_AUTOMATIC:
 		source->priv->automatic = g_value_get_boolean (value);
 		break;
+	case PROP_LOCAL:
+		source->priv->is_local = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -408,7 +463,12 @@ rb_playlist_source_get_property (GObject *object,
 	case PROP_DIRTY:
 		g_value_set_boolean (value, source->priv->dirty);
 		break;
-
+	case PROP_QUERY_MODEL:
+		g_value_set_object (value, source->priv->model);
+		break;
+	case PROP_LOCAL:
+		g_value_set_boolean (value, source->priv->is_local);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -416,7 +476,7 @@ rb_playlist_source_get_property (GObject *object,
 }
 
 RBSource *
-rb_playlist_source_new (RBShell *shell, gboolean automatic)
+rb_playlist_source_new (RBShell *shell, gboolean automatic, gboolean local)
 {
 	RBSource *source;
 	
@@ -424,6 +484,7 @@ rb_playlist_source_new (RBShell *shell, gboolean automatic)
 					  "name", _("Unknown"),
 					  "shell", shell,
 					  "automatic", automatic,
+					  "is-local", local,
 					  NULL));
 
 	return source;
@@ -445,52 +506,47 @@ rb_playlist_source_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry,
 	}
 }
 
-RhythmDBQueryModel *
-rb_playlist_source_get_model (RBPlaylistSource *source)
-{
-	return source->priv->model;
-}
-
 void
 rb_playlist_source_set_query (RBPlaylistSource *source,
 			      GPtrArray *query,
 			      guint limit_count,
-			      guint limit_mb)
+			      guint limit_mb,
+			      guint limit_time,
+			      const char *sort_key,
+			      gint sort_direction)
 {
-	RhythmDBQueryModel *query_model;
-	GtkTreeModel *model;
-	
 	g_assert (source->priv->automatic);
 
-	source->priv->model = query_model
-		= g_object_new (RHYTHMDB_TYPE_QUERY_MODEL,
-				"db", source->priv->db,
-				"max-count", limit_count,
-				"max-size", limit_mb, 
-				NULL);
+	source->priv->query_resetting = TRUE;
 
-	model = GTK_TREE_MODEL (query_model);
+	/* playlists that aren't limited, with a particular sort order, are user-orderable */
+	rb_entry_view_set_columns_clickable (source->priv->songs, (limit_count == 0 && limit_mb == 0));
+	rb_entry_view_set_sorting_order (source->priv->songs, sort_key, sort_direction);
 
-	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_QUERY_MODEL (query_model));
-
-	rhythmdb_do_full_query_async_parsed (source->priv->db, model, query);
-
-	g_object_unref (G_OBJECT (query_model));
-	rb_entry_view_poll_model (source->priv->songs);
+	rb_playlist_source_do_query (source, query, limit_count, limit_mb, limit_time);
+	rhythmdb_query_free (query);
+	source->priv->query_resetting = FALSE;
 }
 
 void
 rb_playlist_source_get_query (RBPlaylistSource *source,
 			      GPtrArray **query,
 			      guint *limit_count,
-			      guint *limit_mb)
+			      guint *limit_mb,
+			      guint *limit_time,
+			      const char **sort_key,
+			      gint *sort_direction)
 {
 	g_assert (source->priv->automatic);
 
 	g_object_get (G_OBJECT (source->priv->model),
 		      "query", query,
 		      "max-count", limit_count,
-		      "max-size", limit_mb, NULL);
+		      "max-size", limit_mb,
+		      "max-time", limit_time,
+		      NULL);
+
+	rb_entry_view_get_sorting_order (source->priv->songs, sort_key, sort_direction);
 }
 
 static char *
@@ -613,22 +669,40 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
                         return FALSE;
 
 	} else {
-		GPtrArray *query;
-		GPtrArray *subquery;
+		GPtrArray *subquery = NULL;
+		gchar **names = g_strsplit ((char *)data->data, "\r\n", 0);
+		guint propid = rb_playlist_source_drag_atom_to_prop (data->type);
+		int i;
 
-		subquery = rhythmdb_query_parse (source->priv->db,
-						 RHYTHMDB_QUERY_PROP_EQUALS,
-						 rb_playlist_source_drag_atom_to_prop (data->type),
-						 data->data,
-						 RHYTHMDB_QUERY_END);
-		query = rhythmdb_query_parse (source->priv->db,
-					      RHYTHMDB_QUERY_PROP_EQUALS,
-					      RHYTHMDB_PROP_TYPE,
-					      RHYTHMDB_ENTRY_TYPE_SONG,
-					      RHYTHMDB_QUERY_SUBQUERY,
-					      subquery,
-					      RHYTHMDB_QUERY_END);
-                rb_playlist_source_set_query (source, query, 0, 0);
+		for (i=0; names[i]; i++) {
+			if (subquery == NULL) 
+				subquery = rhythmdb_query_parse (source->priv->db,
+								 RHYTHMDB_QUERY_PROP_EQUALS,
+								 propid,
+								 names[i],
+								 RHYTHMDB_QUERY_END);
+			else
+				rhythmdb_query_append (source->priv->db,
+						       subquery,
+						       RHYTHMDB_QUERY_DISJUNCTION,
+						       RHYTHMDB_QUERY_PROP_EQUALS,
+						       propid,
+						       names[i],
+						       RHYTHMDB_QUERY_END);
+		}
+
+		g_strfreev (names);
+
+		if (subquery) {
+			GPtrArray *query = rhythmdb_query_parse (source->priv->db,
+							         RHYTHMDB_QUERY_PROP_EQUALS,
+							         RHYTHMDB_PROP_TYPE,
+							         RHYTHMDB_ENTRY_TYPE_SONG,
+							         RHYTHMDB_QUERY_SUBQUERY,
+							         subquery,
+							         RHYTHMDB_QUERY_END);
+			rb_playlist_source_set_query (source, query, 0, 0, 0, NULL, 0);
+		}
 	}
 
         return TRUE;
@@ -673,31 +747,61 @@ rb_playlist_source_drop_cb (GtkWidget *widget,
 	gtk_drag_finish (context, TRUE, FALSE, time);
 }
 
+
+
+static void
+rb_playlist_source_add_location_swapped (const char *uri, RBPlaylistSource *source)
+{
+	rb_playlist_source_add_location (source, uri);
+}
+
 void
 rb_playlist_source_add_location (RBPlaylistSource *source,
 				 const char *location)
 {
 	RhythmDBEntry *entry;
 
-	if (g_hash_table_lookup (source->priv->entries, location)) {
-		return;
-	}
+	if (rb_uri_is_directory (location)) {
+		rb_uri_handle_recursively (location,
+					   (GFunc) rb_playlist_source_add_location_swapped,
+					   NULL,
+					   source);
+	} else {
+		if (g_hash_table_lookup (source->priv->entries, location)) {
+			return;
+		}
 
-	g_hash_table_insert (source->priv->entries,
-			     g_strdup (location), GINT_TO_POINTER (1));
+		g_hash_table_insert (source->priv->entries,
+				     g_strdup (location), GINT_TO_POINTER (1));
 
-	entry = rhythmdb_entry_lookup_by_location (source->priv->db, location);
-	if (entry)
-		rhythmdb_entry_ref (source->priv->db, entry);
+		entry = rhythmdb_entry_lookup_by_location (source->priv->db, location);
+		if (entry)
+			rhythmdb_entry_ref (source->priv->db, entry);
 
-	if (entry != NULL) {
-		rhythmdb_query_model_add_entry (source->priv->model, entry);
-		rhythmdb_entry_unref (source->priv->db, entry);
+		if (entry != NULL) {
+			rhythmdb_query_model_add_entry (source->priv->model, entry);
+			rhythmdb_entry_unref (source->priv->db, entry);
 
-		source->priv->dirty = TRUE;
+			source->priv->dirty = TRUE;
+		}
 	}
 }
 
+void
+rb_playlist_source_add_locations (RBPlaylistSource *source,
+				  GList *locations)
+{
+	GList *l;
+
+	for (l = locations; l; l = l->next) {
+		const gchar *uri = (const gchar *)l->data;
+		
+		rb_playlist_source_add_location (RB_PLAYLIST_SOURCE (source), 
+						 uri);
+	}
+}
+
+
 void
 rb_playlist_source_remove_location (RBPlaylistSource *source,
 				    const char *location)
@@ -773,13 +877,17 @@ rb_playlist_source_save_playlist (RBPlaylistSource *source, const char *uri)
 {
 	TotemPlParser *playlist;
 	GError *error = NULL;
-	rb_debug ("saving playlist");
+	char *name;
 
+
+	rb_debug ("saving playlist");
 	playlist = totem_pl_parser_new ();
 
-	totem_pl_parser_write (playlist, GTK_TREE_MODEL (source->priv->model),
-			       playlist_iter_func, uri, TOTEM_PL_PARSER_PLS,
-			       NULL, &error);
+	g_object_get (G_OBJECT (source), "name", &name, NULL);
+
+	totem_pl_parser_write_with_title (playlist, GTK_TREE_MODEL (source->priv->model),
+					  playlist_iter_func, uri, name,
+					  TOTEM_PL_PARSER_PLS, NULL, &error);
 	if (error != NULL)
 		rb_error_dialog (NULL, _("Couldn't save playlist"),
 				 "%s", error->message);
@@ -806,13 +914,20 @@ rb_playlist_source_burn_playlist (RBPlaylistSource *source)
 #if defined(WITH_CD_BURNER_SUPPORT)
 	GtkWidget *recorder;
 	char *name;
+	RBShell *shell;
+
+	/* don't burn if the playlist is empty */
+	if (gtk_tree_model_iter_n_children (GTK_TREE_MODEL (source->priv->model), NULL) == 0)
+		return;
 
 	rb_debug ("burning playlist");
 
-	g_object_get (source, "name", &name, NULL);
+	g_object_get (source, "name", &name, "shell", &shell, NULL);
 
 	recorder = rb_playlist_source_recorder_new (gtk_widget_get_toplevel (GTK_WIDGET (source)),
+						    shell,
 						    name);
+	g_object_unref (shell);
 	g_free (name);
 
 	rb_playlist_source_recorder_add_from_model (RB_PLAYLIST_SOURCE_RECORDER (recorder),
@@ -837,7 +952,7 @@ rb_playlist_source_new_from_xml	(RBShell *shell,
 	xmlNodePtr child;
 	xmlChar *tmp;
 
-	source = RB_PLAYLIST_SOURCE (rb_playlist_source_new (shell, FALSE));
+	source = RB_PLAYLIST_SOURCE (rb_playlist_source_new (shell, FALSE, TRUE));
 
 	tmp = xmlGetProp (node, RB_PLAYLIST_TYPE);
 	if (!xmlStrcmp (tmp, RB_PLAYLIST_AUTOMATIC))
@@ -848,43 +963,55 @@ rb_playlist_source_new_from_xml	(RBShell *shell,
 	g_object_set (G_OBJECT (source), "name", tmp, NULL);
 	g_free (tmp);
 
-	tmp = xmlGetProp (node, RB_PLAYLIST_INT_NAME);
-	if (!tmp) {
-		GTimeVal serial;
-		/* Hm.  Upgrades. */
-		g_get_current_time (&serial);
-		tmp = (xmlChar *) g_strdup_printf ("<playlist:%ld:%ld>",
-				       serial.tv_sec, serial.tv_usec);
-	}
-	g_object_set (G_OBJECT (source), "internal-name", tmp, NULL);
-	g_free (tmp);
-
 	if (source->priv->automatic) {
 		GPtrArray *query;
-		xmlChar *limit_str;
-		gint limit_count = 0;
-		gint limit_mb = 0;
+		gint limit_count = 0, limit_mb = 0, limit_time = 0;
+		gchar *sort_key = NULL;
+		gint sort_direction = 0;
 
 		child = node->children;
 		while (xmlNodeIsText (child))
 			child = child->next;
 
 		query = rhythmdb_query_deserialize (source->priv->db, child);
-		limit_str = xmlGetProp (node, RB_PLAYLIST_LIMIT_COUNT);
-		if (!limit_str) /* Backwards compatibility */
-			limit_str = xmlGetProp (node, RB_PLAYLIST_LIMIT);
-		if (limit_str) {
-			limit_count = atoi ((char *) limit_str);
-			g_free (limit_str);
+		tmp = xmlGetProp (node, RB_PLAYLIST_LIMIT_COUNT);
+		if (!tmp) /* Backwards compatibility */
+			tmp = xmlGetProp (node, RB_PLAYLIST_LIMIT);
+		if (tmp) {
+			limit_count = atoi ((char*) tmp);
+			g_free (tmp);
 		}
-		limit_str = xmlGetProp (node, RB_PLAYLIST_LIMIT_SIZE);
-		if (limit_str) {
-			limit_mb = atoi ((char *) limit_str);
-			g_free (limit_str);
+		tmp = xmlGetProp (node, RB_PLAYLIST_LIMIT_SIZE);
+		if (tmp) {
+			limit_mb = atoi ((char*) tmp);
+			g_free (tmp);
 		}
+		tmp = xmlGetProp (node, RB_PLAYLIST_LIMIT_TIME);
+		if (tmp) {
+			limit_time = atoi ((char*) tmp);
+			g_free (tmp);
+		}
+
+		sort_key = (gchar*) xmlGetProp (node, RB_PLAYLIST_SORT_KEY);
+		if (sort_key && *sort_key) {
+			tmp = xmlGetProp (node, RB_PLAYLIST_SORT_DIRECTION);
+			if (tmp) {
+				sort_direction = atoi ((char*) tmp);
+				g_free (tmp);
+			}
+		} else {
+			g_free (sort_key);
+			sort_key = NULL;
+			sort_direction = 0;
+		}
+
 		rb_playlist_source_set_query (source, query,
 					      limit_count,
-					      limit_mb);
+					      limit_mb,
+					      limit_time,
+					      sort_key,
+					      sort_direction);
+		g_free (sort_key);
 	} else {
 		for (child = node->children; child; child = child->next) {
 			xmlChar *location;
@@ -909,14 +1036,11 @@ rb_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr parent_node
 {
 	xmlNodePtr node;
 	xmlChar *name;
-	xmlChar *internal_name;
 	GtkTreeIter iter;
 
 	node = xmlNewChild (parent_node, NULL, RB_PLAYLIST_PLAYLIST, NULL);
-	g_object_get (G_OBJECT (source), "name", &name,
-		      "internal-name", &internal_name, NULL);
+	g_object_get (G_OBJECT (source), "name", &name, NULL);
 	xmlSetProp (node, RB_PLAYLIST_NAME, name);
-	xmlSetProp (node, RB_PLAYLIST_INT_NAME, internal_name);
 	xmlSetProp (node, RB_PLAYLIST_TYPE, source->priv->automatic ? RB_PLAYLIST_AUTOMATIC : RB_PLAYLIST_STATIC);
 
 	if (!source->priv->automatic) {
@@ -938,22 +1062,103 @@ rb_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr parent_node
 						   &iter));
 	} else {
 		GPtrArray *query;
-		guint max_count;
-		guint max_size_mb;
-		char *limit_str;
+		guint max_count, max_size_mb, max_time;
+		const gchar *sort_key;
+		gint sort_direction;
+		gchar *temp_str;
+
+		rb_playlist_source_get_query (source,
+					      &query,
+					      &max_count, &max_size_mb, &max_time,
+					      &sort_key, &sort_direction);
+		temp_str = g_strdup_printf ("%d", max_count);
+		xmlSetProp (node, RB_PLAYLIST_LIMIT_COUNT, BAD_CAST temp_str);
+		g_free (temp_str);
+		temp_str = g_strdup_printf ("%d", max_size_mb);
+		xmlSetProp (node, RB_PLAYLIST_LIMIT_SIZE, BAD_CAST temp_str);
+		g_free (temp_str);
+		temp_str = g_strdup_printf ("%d", max_time);
+		xmlSetProp (node, RB_PLAYLIST_LIMIT_TIME, BAD_CAST temp_str);
+		g_free (temp_str);
+
+		if (sort_key && *sort_key) {
+			xmlSetProp (node, RB_PLAYLIST_SORT_KEY, BAD_CAST sort_key);
+			temp_str = g_strdup_printf ("%d", sort_direction);
+			xmlSetProp (node, RB_PLAYLIST_SORT_DIRECTION, BAD_CAST temp_str);
+			g_free (temp_str);
+		}
 
-		g_object_get (G_OBJECT (source->priv->model),
-			      "max-count", &max_count,
-			      "max-size", &max_size_mb,
-			      "query", &query, NULL);
-		limit_str = g_strdup_printf ("%d", max_count);
-		xmlSetProp (node, RB_PLAYLIST_LIMIT_COUNT, BAD_CAST limit_str);
-		g_free (limit_str);
-		limit_str = g_strdup_printf ("%d", max_size_mb);
-		xmlSetProp (node, RB_PLAYLIST_LIMIT_SIZE, BAD_CAST limit_str);
-		g_free (limit_str);
 		rhythmdb_query_serialize (source->priv->db, query, node);
 	}
 
 	source->priv->dirty = FALSE;
 }
+
+static void
+rb_playlist_source_songs_sort_order_changed_cb (RBEntryView *view, RBPlaylistSource *source)
+{
+	GPtrArray *query;
+	guint limit_count, limit_mb, limit_time;
+
+	g_assert (source->priv->automatic);
+
+	/* don't process this if we are in the middle of setting a query */
+	if (source->priv->query_resetting)
+		return;
+	rb_debug ("sort order changed");
+
+	/* need to re-run query with the same settings*/
+	g_object_get (G_OBJECT (source->priv->model),
+		      "query", &query,
+		      "max-count", &limit_count,
+		      "max-size", &limit_mb,
+		      "max-size", &limit_time,
+		      NULL);
+
+	rb_playlist_source_do_query (source, query, limit_count, limit_mb, limit_time);
+	rhythmdb_query_free (query);
+}
+
+static void
+rb_playlist_source_do_query (RBPlaylistSource *source,
+			      GPtrArray *query,
+			      guint limit_count,
+			      guint limit_mb,
+			      guint limit_time)
+{
+	g_assert (source->priv->automatic);
+
+	source->priv->model = g_object_new (RHYTHMDB_TYPE_QUERY_MODEL,
+				    "db", source->priv->db,
+				    "max-count", limit_count,
+				    "max-size", limit_mb, 
+				    "max-time", limit_time, 
+				    NULL);
+
+	rb_entry_view_set_model (source->priv->songs, source->priv->model);
+	rhythmdb_do_full_query_async_parsed (source->priv->db, GTK_TREE_MODEL (source->priv->model), query);
+
+	/* emit notification the the property has changed */
+	g_object_notify (G_OBJECT (source), "query-model");
+}
+
+static void
+rb_playlist_source_non_entry_dropped (GtkTreeModel *model,
+				      const char *uri,
+				      RBPlaylistSource *source)
+{
+	rb_playlist_source_add_location (source, uri);
+}
+
+static void
+rb_playlist_source_row_inserted (GtkTreeModel *model,
+				 GtkTreePath *path,
+				 GtkTreeIter *iter,
+				 RBPlaylistSource *source)
+{
+	RhythmDBEntry *entry;
+
+	gtk_tree_model_get (model, iter, 0, &entry, -1);
+
+	rb_playlist_source_add_location (source, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+}
