@@ -46,24 +46,30 @@ static void rb_source_get_property (GObject *object,
 					GParamSpec *pspec);
 
 static const char * default_get_browser_key (RBSource *status);
-static const char * default_get_search_key (RBSource *status);
 static GList *default_get_extra_views (RBSource *source);
 static gboolean default_can_rename (RBSource *source);
 static gboolean default_can_search (RBSource *source);
 static gboolean default_can_cut (RBSource *source);
+static GdkPixbuf *default_get_pixbuf (RBSource *source);
 static GList *default_copy (RBSource *source);
 static void default_reset_filters (RBSource *source);
 static void default_song_properties (RBSource *source);
 static GtkWidget * default_get_config_widget (RBSource *source);
+static gboolean default_try_playlist (RBSource *source);
 static RBSourceEOFType default_handle_eos (RBSource *source);
 static gboolean default_receive_drag  (RBSource *source, GtkSelectionData *data);
 static gboolean default_show_popup  (RBSource *source);
 static void default_delete_thyself (RBSource *source);
+static void default_activate (RBSource *source);
+static void default_deactivate (RBSource *source);
+static gboolean default_disconnect (RBSource *source);
 
-struct RBSourcePrivate
+G_DEFINE_ABSTRACT_TYPE (RBSource, rb_source, GTK_TYPE_HBOX)
+#define RB_SOURCE_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), RB_TYPE_SOURCE, RBSourcePrivate))
+
+struct _RBSourcePrivate
 {
 	char *name;
-	char *internal_name;
 	
 	RBShell *shell;
 	gboolean visible;
@@ -73,7 +79,6 @@ enum
 {
 	PROP_0,
 	PROP_NAME,
-	PROP_INTERNAL_NAME,
 	PROP_SHELL,
 	PROP_UI_MANAGER,
 	PROP_VISIBLE
@@ -89,56 +94,24 @@ enum
 
 static guint rb_source_signals[LAST_SIGNAL] = { 0 };
 
-static GObjectClass *parent_class = NULL;
-
-GType
-rb_source_get_type (void)
-{
-	static GType rb_source_type = 0;
-
-	if (rb_source_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBSourceClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_source_class_init,
-			NULL,
-			NULL,
-			sizeof (RBSource),
-			0,
-			(GInstanceInitFunc) rb_source_init
-		};
-
-		rb_source_type = g_type_register_static (GTK_TYPE_HBOX,
-						       "RBSource",
-						       &our_info, G_TYPE_FLAG_ABSTRACT);
-	}
-
-	return rb_source_type;
-}
-
 static void
 rb_source_class_init (RBSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = rb_source_finalize;
 
 	object_class->set_property = rb_source_set_property;
 	object_class->get_property = rb_source_get_property;
 
 	klass->impl_get_browser_key = default_get_browser_key;
-	klass->impl_get_search_key = default_get_search_key;
 	klass->impl_get_extra_views = default_get_extra_views;
 	klass->impl_can_rename = default_can_rename;
 	klass->impl_can_search = default_can_search;
 	klass->impl_can_cut = default_can_cut;
 	klass->impl_can_delete = default_can_cut;
 	klass->impl_can_copy = default_can_cut;
+	klass->impl_get_pixbuf = default_get_pixbuf;
 	klass->impl_copy = default_copy;
 	klass->impl_reset_filters = default_reset_filters;
 	klass->impl_song_properties = default_song_properties;
@@ -147,6 +120,10 @@ rb_source_class_init (RBSourceClass *klass)
 	klass->impl_receive_drag = default_receive_drag;
 	klass->impl_show_popup = default_show_popup;
 	klass->impl_delete_thyself = default_delete_thyself;
+	klass->impl_activate = default_activate;
+	klass->impl_deactivate = default_deactivate;
+	klass->impl_disconnect = default_disconnect;
+	klass->impl_try_playlist = default_try_playlist;
 
 	g_object_class_install_property (object_class,
 					 PROP_NAME,
@@ -155,13 +132,6 @@ rb_source_class_init (RBSourceClass *klass)
 							      "Interface name",
 							      NULL,
 							      G_PARAM_READWRITE));
-	g_object_class_install_property (object_class,
-					 PROP_INTERNAL_NAME,
-					 g_param_spec_string ("internal-name",
-							      "Internal name",
-							      "Internal name",
-							      NULL,
-							      G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 					 PROP_SHELL,
@@ -198,7 +168,7 @@ rb_source_class_init (RBSourceClass *klass)
 		g_signal_new ("deleted",
 			      RB_TYPE_SOURCE,
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (RBSourceClass, status_changed),
+			      G_STRUCT_OFFSET (RBSourceClass, deleted),
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
@@ -214,7 +184,7 @@ rb_source_class_init (RBSourceClass *klass)
 			      G_TYPE_NONE,
 			      0);
 
-	/**
+	/*
 	 * Fires when the user changes the filter, either by changing the
 	 * contents of the search box or by selecting a different browser
 	 * entry.
@@ -228,34 +198,34 @@ rb_source_class_init (RBSourceClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	
+	g_type_class_add_private (object_class, sizeof (RBSourcePrivate));
 }
 
 static void
 rb_source_init (RBSource *source)
 {
-	source->priv = g_new0 (RBSourcePrivate, 1);
-	source->priv->visible = TRUE;
+	RB_SOURCE_GET_PRIVATE (source)->visible = TRUE;
 }
 
 static void
 rb_source_finalize (GObject *object)
 {
 	RBSource *source;
-
+	RBSourcePrivate *priv;
+	
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (RB_IS_SOURCE (object));
 
 	source = RB_SOURCE (object);
-
-	g_return_if_fail (source->priv != NULL);
+	priv = RB_SOURCE_GET_PRIVATE (source);
+	g_return_if_fail (priv != NULL);
 
 	rb_debug ("Finalizing view %p", source);
 
-	g_free (source->priv->name);
+	g_free (priv->name);
 
-	g_free (source->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_source_parent_class)->finalize (object);
 }
 
 static void
@@ -264,26 +234,22 @@ rb_source_set_property (GObject *object,
 		      const GValue *value,
 		      GParamSpec *pspec)
 {
-	RBSource *source = RB_SOURCE (object);
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (object);
 
 	switch (prop_id)
 	{
 	case PROP_NAME:
-		g_free (source->priv->name);
-		source->priv->name = g_strdup (g_value_get_string (value));
-		break;
-	case PROP_INTERNAL_NAME:
-		g_free (source->priv->internal_name);
-		source->priv->internal_name = g_strdup (g_value_get_string (value));
+		g_free (priv->name);
+		priv->name = g_strdup (g_value_get_string (value));
 		break;
 	case PROP_SHELL:
-		source->priv->shell = g_value_get_object (value);
+		priv->shell = g_value_get_object (value);
 		break;
 	case PROP_VISIBLE:
-		source->priv->visible = g_value_get_boolean (value);
-		rb_debug ("Setting %s visibility to %u\n", 
-			  source->priv->name, 
-			  source->priv->visible);
+		priv->visible = g_value_get_boolean (value);
+		rb_debug ("Setting %s visibility to %u", 
+			  priv->name, 
+			  priv->visible);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -297,26 +263,23 @@ rb_source_get_property (GObject *object,
 		      GValue *value,
 		      GParamSpec *pspec)
 {
-	RBSource *source = RB_SOURCE (object);
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (object);
 
 	switch (prop_id)
 	{
 	case PROP_NAME:
-		g_value_set_string (value, source->priv->name);
-		break;
-	case PROP_INTERNAL_NAME:
-		g_value_set_string (value, source->priv->internal_name);
+		g_value_set_string (value, priv->name);
 		break;
 	case PROP_SHELL:
-		g_value_set_object (value, source->priv->shell);
+		g_value_set_object (value, priv->shell);
 		break;
 	case PROP_VISIBLE:
-		g_value_set_boolean (value, source->priv->visible);
+		g_value_set_boolean (value, priv->visible);
 		break;
 	case PROP_UI_MANAGER:
 	{
 		GtkUIManager *manager;
-		g_object_get (G_OBJECT (source->priv->shell), 
+		g_object_get (G_OBJECT (priv->shell), 
 			      "ui-manager", &manager,
 			      NULL);
 		g_value_set_object (value, manager);
@@ -329,6 +292,15 @@ rb_source_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_source_get_status:
+ * @status: a #RBSource
+ *
+ * FIXME:
+ * Some Random comments
+ *
+ * Returns: The status string
+ **/
 char *
 rb_source_get_status (RBSource *status)
 {
@@ -343,12 +315,6 @@ default_get_browser_key (RBSource *status)
 	return NULL;
 }
 
-static const char *
-default_get_search_key (RBSource *status)
-{
-	return NULL;
-}
-
 const char *
 rb_source_get_browser_key (RBSource *status)
 {
@@ -357,14 +323,6 @@ rb_source_get_browser_key (RBSource *status)
 	return klass->impl_get_browser_key (status);
 }
 
-const char *
-rb_source_get_search_key (RBSource *status)
-{
-	RBSourceClass *klass = RB_SOURCE_GET_CLASS (status);
-
-	return klass->impl_get_search_key (status);
-}
-
 void
 rb_source_notify_status_changed (RBSource *status)
 {
@@ -391,7 +349,7 @@ rb_source_update_play_statistics (RBSource *source, RhythmDB *db, RhythmDBEntry
 	g_value_set_ulong (&value, current_count + 1);
 
 	/* Increment current play count */
-	rhythmdb_entry_sync (db, entry, RHYTHMDB_PROP_PLAY_COUNT, &value);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_PLAY_COUNT, &value);
 	g_value_unset (&value);
 	
 	/* Reset the last played time */
@@ -399,7 +357,7 @@ rb_source_update_play_statistics (RBSource *source, RhythmDB *db, RhythmDBEntry
 
 	g_value_init (&value, G_TYPE_ULONG);
 	g_value_set_ulong (&value, now);
-	rhythmdb_entry_sync (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &value);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &value);
 	g_value_unset (&value);
 
 	rhythmdb_commit (db);
@@ -427,6 +385,12 @@ rb_source_get_extra_views (RBSource *source)
 	return klass->impl_get_extra_views (source);
 }
 
+static GdkPixbuf *
+default_get_pixbuf (RBSource *source)
+{
+	return NULL;
+}
+
 GdkPixbuf *
 rb_source_get_pixbuf (RBSource *source)
 {
@@ -468,7 +432,9 @@ rb_source_search (RBSource *source, const char *text)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
-	klass->impl_search (source, text);
+	/* several sources don't have a search ability */
+	if (klass->impl_search != NULL)
+		klass->impl_search (source, text);
 }
 
 static GtkWidget *
@@ -482,7 +448,11 @@ rb_source_get_config_widget (RBSource *source)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
-	return klass->impl_get_config_widget (source);
+	if (klass->impl_get_config_widget) {
+		return klass->impl_get_config_widget (source);
+	} else {
+		return NULL;
+	}
 }
 
 static gboolean
@@ -581,6 +551,14 @@ rb_source_song_properties (RBSource *source)
 	klass->impl_song_properties (source);
 }
 
+gboolean
+rb_source_try_playlist (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+
+	return klass->impl_try_playlist (source);
+}
+
 gboolean
 rb_source_can_pause (RBSource *source)
 {
@@ -589,6 +567,12 @@ rb_source_can_pause (RBSource *source)
 	return klass->impl_can_pause (source);
 }
 
+static gboolean
+default_try_playlist (RBSource *source)
+{
+	return FALSE;
+}
+
 static RBSourceEOFType
 default_handle_eos (RBSource *source)
 {
@@ -603,30 +587,6 @@ rb_source_handle_eos (RBSource *source)
 	return klass->impl_handle_eos (source);
 }
 
-gboolean
-rb_source_have_artist_album (RBSource *source)
-{
-	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
-
-	return klass->impl_have_artist_album (source);
-}
-
-const char *
-rb_source_get_artist (RBSource *source)
-{
-	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
-
-	return klass->impl_get_artist (source);
-}
-
-const char *
-rb_source_get_album (RBSource *source)
-{
-	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
-
-	return klass->impl_get_album (source);
-}
-
 gboolean
 rb_source_have_url (RBSource *source)
 {
@@ -657,7 +617,7 @@ _rb_source_show_popup (RBSource *source, const char *ui_path)
 {
 	GtkUIManager *uimanager;
 
-	g_object_get (G_OBJECT (source->priv->shell), 
+	g_object_get (G_OBJECT (RB_SOURCE_GET_PRIVATE (source)->shell), 
 		      "ui-manager", &uimanager, NULL);
 	rb_gtk_action_popup_menu (uimanager, ui_path);
 	g_object_unref (G_OBJECT (uimanager));
@@ -692,3 +652,44 @@ rb_source_delete_thyself (RBSource *source)
 	klass->impl_delete_thyself (source);
 	g_signal_emit (G_OBJECT (source), rb_source_signals[DELETED], 0);
 }
+
+static void default_activate (RBSource *source)
+{
+	return;
+}
+
+static void default_deactivate (RBSource *source)
+{
+	return;
+}
+
+static gboolean default_disconnect (RBSource *source)
+{
+	return TRUE;
+}
+
+void rb_source_activate (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+
+	klass->impl_activate (source);
+
+	return;
+}
+
+void rb_source_deactivate (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+
+	klass->impl_deactivate (source);
+
+	return;
+}
+
+gboolean rb_source_disconnect (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+
+	return klass->impl_disconnect (source);
+}
+
