@@ -47,8 +47,8 @@ struct RBSourceListPriv
 	GtkTreeSelection *selection;
 
 	RBSource *playing_source;
-	GdkPixbuf *playing_pixbuf;
-	GdkPixbuf *paused_pixbuf;
+	int child_source_count;
+	GtkTreeViewColumn *expander_column;
 
 	RBShell *shell;
 };
@@ -56,7 +56,8 @@ struct RBSourceListPriv
 enum
 {
 	PROP_0,
-	PROP_SHELL
+	PROP_SHELL,
+	PROP_MODEL
 };
 
 enum
@@ -79,6 +80,12 @@ static void rb_sourcelist_get_property (GObject *object,
 					guint prop_id,
 					GValue *value,
 					GParamSpec *pspec);
+static gboolean rb_sourcelist_source_to_iter (RBSourceList *sourcelist, 
+					      RBSource *source,
+					      GtkTreeIter *iter);
+static gboolean rb_sourcelist_visible_source_to_iter (RBSourceList *sourcelist, 
+						      RBSource *source,
+						      GtkTreeIter *iter);
 static void rb_sourcelist_selection_changed_cb (GtkTreeSelection *selection,
 						RBSourceList *sourcelist);
 static void drop_received_cb (RBSourceListModel *model, RBSource *target, GtkTreeViewDropPosition pos,
@@ -94,12 +101,10 @@ static void visibility_notify_cb (GObject *obj, GParamSpec *pspec,
 				  gpointer data);
 static void source_name_edited_cb (GtkCellRendererText *renderer, const char *pathstr,
 				   const char *text, RBSourceList *sourcelist);
-static void rb_sourcelist_playing_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
-						  GtkTreeModel *tree_model, GtkTreeIter *iter,
-						  RBSourceList *sourcelist);
 static void rb_sourcelist_title_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
 						GtkTreeModel *tree_model, GtkTreeIter *iter,
 						RBSourceList *sourcelist);
+static void rb_sourcelist_update_expander_visibility (RBSourceList *sourcelist);
 
 static GtkVBoxClass *parent_class = NULL;
 
@@ -154,6 +159,13 @@ rb_sourcelist_class_init (RBSourceListClass *class)
 							      RB_TYPE_SHELL,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property (o_class,
+					 PROP_MODEL,
+					 g_param_spec_object ("model",
+							      "GtkTreeModel",
+							      "GtkTreeModel object",
+							      GTK_TYPE_TREE_MODEL,
+							      G_PARAM_READABLE));
 	rb_sourcelist_signals[SELECTED] =
 		g_signal_new ("selected",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -204,7 +216,6 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *gcolumn;
-	GtkWidget *dummy;
 
 	sourcelist->priv = g_new0 (RBSourceListPriv, 1);
 
@@ -218,6 +229,8 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 
 	sourcelist->priv->treeview = gtk_tree_view_new_with_model (sourcelist->priv->filter_model);
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (sourcelist->priv->treeview), FALSE);
+	rb_sourcelist_model_set_dnd_targets (RB_SOURCELIST_MODEL (sourcelist->priv->filter_model),
+					     GTK_TREE_VIEW (sourcelist->priv->treeview));
 
 	g_signal_connect_object (G_OBJECT (sourcelist->priv->treeview),
 				 "row_activated",
@@ -234,20 +247,14 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 				 G_CALLBACK (popup_menu_cb),
 				 sourcelist, 0);
 
-	dummy = gtk_tree_view_new ();
-	sourcelist->priv->playing_pixbuf = gtk_widget_render_icon (dummy,
-								   RB_STOCK_PLAYING,
-								   GTK_ICON_SIZE_MENU,
-								   NULL);
-
-	sourcelist->priv->paused_pixbuf = gtk_widget_render_icon (dummy,
-								  RB_STOCK_PAUSED,
-								  GTK_ICON_SIZE_MENU,
-								  NULL);
-	gtk_widget_destroy (dummy);
+	sourcelist->priv->expander_column = gtk_tree_view_column_new ();
+	gtk_tree_view_append_column (GTK_TREE_VIEW (sourcelist->priv->treeview),
+				     sourcelist->priv->expander_column);
+	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (sourcelist->priv->treeview),
+				     sourcelist->priv->expander_column);
 
 	gcolumn = gtk_tree_view_column_new ();
-	gtk_tree_view_column_set_title (gcolumn, _("_Source"));
+	gtk_tree_view_column_set_title (gcolumn, _("S_ource"));
 	gtk_tree_view_column_set_clickable (gcolumn, FALSE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (sourcelist->priv->treeview), gcolumn);
 
@@ -267,20 +274,14 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 						 sourcelist, NULL);
 	g_signal_connect_object (renderer, "edited", G_CALLBACK (source_name_edited_cb), sourcelist, 0);
 
-	/* Set up the playing column */
-	renderer = rb_cell_renderer_pixbuf_new ();
-	gtk_tree_view_column_pack_start (gcolumn, renderer, FALSE);
-	gtk_tree_view_column_set_cell_data_func (gcolumn, renderer,
-						 (GtkTreeCellDataFunc)
-						 rb_sourcelist_playing_cell_data_func,
-						 sourcelist, NULL);
-
 	sourcelist->priv->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sourcelist->priv->treeview));
 	g_signal_connect_object (G_OBJECT (sourcelist->priv->selection),
 			         "changed",
 			         G_CALLBACK (rb_sourcelist_selection_changed_cb),
 			         sourcelist,
 				 0);
+
+	rb_sourcelist_update_expander_visibility (sourcelist);
 }
 
 static void
@@ -288,9 +289,6 @@ rb_sourcelist_finalize (GObject *object)
 {
 	RBSourceList *sourcelist = RB_SOURCELIST (object);
 
-	g_object_unref (G_OBJECT (sourcelist->priv->playing_pixbuf));
-	g_object_unref (G_OBJECT (sourcelist->priv->paused_pixbuf));
-
 	g_free (sourcelist->priv);
 
 	if (G_OBJECT_CLASS (parent_class)->finalize)
@@ -326,6 +324,9 @@ rb_sourcelist_get_property (GObject *object,
 	case PROP_SHELL:
 		g_value_set_object (value, sourcelist->priv->shell);
 		break;
+	case PROP_MODEL:
+		g_value_set_object (value, sourcelist->priv->filter_model);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -355,7 +356,8 @@ rb_sourcelist_new (RBShell *shell)
 
 void
 rb_sourcelist_append (RBSourceList *sourcelist,
-		      RBSource *source)
+		      RBSource *source,
+		      RBSource *parent)
 {
 	GtkTreeIter iter;
 	PangoAttrList *attrs = pango_attr_list_new ();
@@ -366,9 +368,19 @@ rb_sourcelist_append (RBSourceList *sourcelist,
 
 	g_object_get (G_OBJECT (source), "name", &name, NULL);
 
-	gtk_list_store_append (GTK_LIST_STORE (sourcelist->priv->real_model), &iter);
+	if (parent) {
+		GtkTreeIter parent_iter;
+		g_assert (rb_sourcelist_source_to_iter (sourcelist, parent, &parent_iter));
+		gtk_tree_store_append (GTK_TREE_STORE (sourcelist->priv->real_model), &iter, &parent_iter);
 
-	gtk_list_store_set (GTK_LIST_STORE (sourcelist->priv->real_model), &iter,
+		/* !!! make the expander column visible */
+		sourcelist->priv->child_source_count++;
+		rb_sourcelist_update_expander_visibility (sourcelist);
+	} else {
+		gtk_tree_store_append (GTK_TREE_STORE (sourcelist->priv->real_model), &iter, NULL);
+	}
+
+	gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model), &iter,
 			    RB_SOURCELIST_MODEL_COLUMN_PIXBUF, rb_source_get_pixbuf (source),
 			    RB_SOURCELIST_MODEL_COLUMN_NAME, name,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, source,
@@ -378,41 +390,73 @@ rb_sourcelist_append (RBSourceList *sourcelist,
 	g_signal_connect_object (G_OBJECT (source), "notify::name", G_CALLBACK (name_notify_cb), sourcelist, 0);
 	g_signal_connect_object (G_OBJECT (source), "notify::visibility", G_CALLBACK (visibility_notify_cb), sourcelist, 0);
 
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (sourcelist->priv->treeview));
+}
+
+typedef struct _SourcePath {
+	RBSource *source;
+	GtkTreePath *path;
+} SourcePath;
+
+static gboolean
+match_source_to_iter (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
+		      SourcePath *sp)
+{
+	RBSource *target = NULL;
+	
+	gtk_tree_model_get (model, iter, RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
+	if (target == sp->source) {
+		sp->path = gtk_tree_path_copy(path);
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 static gboolean
 rb_sourcelist_source_to_iter (RBSourceList *sourcelist, RBSource *source,
 			      GtkTreeIter *iter)
 {
-	if (gtk_tree_model_get_iter_first (sourcelist->priv->real_model, iter) == FALSE) {
-		return FALSE;
+	SourcePath *sp = g_new0(SourcePath,1);
+	gboolean ret = FALSE;
+	
+	sp->source = source;
+	
+	gtk_tree_model_foreach (sourcelist->priv->real_model, (GtkTreeModelForeachFunc) match_source_to_iter, sp);
+	
+	if (sp->path) {
+		gtk_tree_model_get_iter (sourcelist->priv->real_model, iter, sp->path);
+		ret = TRUE;
 	}
 
-	do {
-		RBSource *target = NULL;
-		gtk_tree_model_get (sourcelist->priv->real_model, iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
-		if (source == target)
-			return TRUE;
-	} while (gtk_tree_model_iter_next (sourcelist->priv->real_model, iter));
-	return FALSE;
+	gtk_tree_path_free(sp->path);
+	g_free(sp);
+	sp = NULL;
+
+	return ret;
 }
 
 static gboolean
 rb_sourcelist_visible_source_to_iter (RBSourceList *sourcelist, RBSource *source,
 				      GtkTreeIter *iter)
 {
-	if (gtk_tree_model_get_iter_first (sourcelist->priv->filter_model, iter) == FALSE) {
-		return FALSE;
+	SourcePath *sp = g_new0(SourcePath,1);
+	gboolean ret = FALSE;
+	
+	sp->source = source;
+	
+	gtk_tree_model_foreach (sourcelist->priv->filter_model, (GtkTreeModelForeachFunc) match_source_to_iter, sp);
+
+	if (sp->path) {
+		gtk_tree_model_get_iter (sourcelist->priv->filter_model, iter, sp->path);
+		ret = TRUE;
 	}
-	do {
-		RBSource *target = NULL;
-		gtk_tree_model_get (sourcelist->priv->filter_model, iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
-		if (source == target)
-			return TRUE;
-	} while (gtk_tree_model_iter_next (sourcelist->priv->filter_model, iter));
-	return FALSE;
+
+	gtk_tree_path_free(sp->path);
+	g_free(sp);
+	sp = NULL;
+
+	return ret;
 }
 
 void
@@ -446,13 +490,13 @@ rb_sourcelist_set_playing_source (RBSourceList *sourcelist, RBSource *source)
 		g_assert (rb_sourcelist_source_to_iter (sourcelist,
 							sourcelist->priv->playing_source,
 							&iter));
-		gtk_list_store_set (GTK_LIST_STORE (sourcelist->priv->real_model), &iter,
+		gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model), &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_PLAYING, FALSE, -1);
 	}
 	sourcelist->priv->playing_source = source;
 	if (source) {
 		g_assert (rb_sourcelist_source_to_iter (sourcelist, source, &iter));
-		gtk_list_store_set (GTK_LIST_STORE (sourcelist->priv->real_model), &iter,
+		gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model), &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_PLAYING, TRUE, -1);
 	}
 }
@@ -463,12 +507,20 @@ rb_sourcelist_remove (RBSourceList *sourcelist, RBSource *source)
 	GtkTreeIter iter;
 
 	g_assert (rb_sourcelist_source_to_iter (sourcelist, source, &iter));
-	gtk_list_store_remove (GTK_LIST_STORE (sourcelist->priv->real_model), &iter);
+
+	if (gtk_tree_store_iter_depth (GTK_TREE_STORE (sourcelist->priv->real_model), &iter) > 0) {
+		sourcelist->priv->child_source_count--;
+		rb_sourcelist_update_expander_visibility (sourcelist);
+	}
+
+	gtk_tree_store_remove (GTK_TREE_STORE (sourcelist->priv->real_model), &iter);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (source),
 					      G_CALLBACK (name_notify_cb), sourcelist);
         g_signal_handlers_disconnect_by_func (G_OBJECT (source),
 					      G_CALLBACK (visibility_notify_cb),
                                              sourcelist);
+
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (sourcelist->priv->treeview));
 }
 
 void
@@ -480,28 +532,6 @@ rb_sourcelist_select (RBSourceList *sourcelist, RBSource *source)
 	gtk_tree_selection_select_iter (sourcelist->priv->selection, &iter);
 }
 
-static void
-rb_sourcelist_playing_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
-				      GtkTreeModel *tree_model, GtkTreeIter *iter,
-				      RBSourceList *sourcelist)
-{
-	GdkPixbuf *pixbuf;
-	gboolean playing;
-	gtk_tree_model_get (sourcelist->priv->filter_model, iter,
-			    RB_SOURCELIST_MODEL_COLUMN_PLAYING, &playing, -1);
-	
-	if (playing)
-		pixbuf = sourcelist->priv->playing_pixbuf;
-#if 0
-	else if (source == sourcelist->priv->playing_source)
-		pixbuf = sourcelist->priv->paused_pixbuf;
-#endif
-	else
-		pixbuf = NULL;
-
-	g_object_set (G_OBJECT (renderer), "pixbuf", pixbuf, NULL);
-}
-
 static void
 rb_sourcelist_selection_changed_cb (GtkTreeSelection *selection,
 				    RBSourceList *sourcelist)
@@ -522,24 +552,6 @@ rb_sourcelist_selection_changed_cb (GtkTreeSelection *selection,
 	g_signal_emit (G_OBJECT (sourcelist), rb_sourcelist_signals[SELECTED], 0, source);
 }
 
-void
-rb_sourcelist_set_dnd_targets (RBSourceList *sourcelist,
-			       const GtkTargetEntry *targets,
-			       int n_targets)
-{
-	g_return_if_fail (RB_IS_SOURCELIST (sourcelist));
-
-	rb_tree_dnd_add_drag_dest_support (GTK_TREE_VIEW (sourcelist->priv->treeview),
-					   RB_TREE_DEST_EMPTY_VIEW_DROP,
-					   targets, n_targets,
-					   GDK_ACTION_LINK);
-	rb_tree_dnd_add_drag_source_support (GTK_TREE_VIEW (sourcelist->priv->treeview),
-					     GDK_BUTTON1_MASK,
-					     targets,
-					     n_targets,
-					     GDK_ACTION_MOVE);
-}
-
 static void
 drop_received_cb (RBSourceListModel *model, RBSource *target, GtkTreeViewDropPosition pos,
 		  GtkSelectionData *data, RBSourceList *sourcelist)
@@ -631,18 +643,33 @@ name_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data)
 	
 	if (rb_sourcelist_source_to_iter (sourcelist, source, &iter)) {
 		g_object_get (obj, "name", &name, NULL);
-		gtk_list_store_set (GTK_LIST_STORE (sourcelist->priv->real_model),
+		gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model),
 				    &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_NAME, name, -1);
 	}
+
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (sourcelist->priv->treeview));
 }
 	
 static void
 visibility_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data)
 {
 	RBSourceList *sourcelist = RB_SOURCELIST (data);
-	
+	RBSource *source = RB_SOURCE (obj);
+	GtkTreeIter iter;
+
 	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (sourcelist->priv->filter_model));
+
+	g_assert (rb_sourcelist_source_to_iter (sourcelist, source, &iter));
+	if (gtk_tree_store_iter_depth (GTK_TREE_STORE (sourcelist->priv->real_model), &iter) > 0) {
+		gboolean visible;
+
+		g_object_get (obj, "visibility", &visible, NULL);
+		sourcelist->priv->child_source_count += visible ? 1 : -1;
+		rb_sourcelist_update_expander_visibility (sourcelist);
+	}
+
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (sourcelist->priv->treeview));
 }
 
 static void
@@ -652,12 +679,17 @@ rb_sourcelist_title_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *
 {
 	RBSource *source;
 	char *str;
-
+	gboolean playing;
+	
 	gtk_tree_model_get (GTK_TREE_MODEL (sourcelist->priv->filter_model), iter,
 			    RB_SOURCELIST_MODEL_COLUMN_NAME, &str,
-			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
+			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source,
+			    RB_SOURCELIST_MODEL_COLUMN_PLAYING, &playing,
+			    -1);
 
-	g_object_set (G_OBJECT (renderer), "text", str, NULL);
+	g_object_set (G_OBJECT (renderer), "text", str,
+		      "weight", playing ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL,
+		      NULL);
 
 	g_free (str);
 }
@@ -684,5 +716,17 @@ source_name_edited_cb (GtkCellRendererText *renderer, const char *pathstr,
 	gtk_tree_path_free (path);
 
 	g_object_set (G_OBJECT (renderer), "editable", FALSE, NULL);
+
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (sourcelist->priv->treeview));
 }
 
+static void
+rb_sourcelist_update_expander_visibility (RBSourceList *sourcelist)
+{
+	gboolean visible;
+
+	g_assert (sourcelist->priv->child_source_count >= 0);
+
+	visible = (sourcelist->priv->child_source_count > 0);
+	gtk_tree_view_column_set_visible (sourcelist->priv->expander_column, visible);
+}
